#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2020-04-29
#
# inventory of Juniper BGP Peers (IPv4 and IPv6)
#
# THX to Jeff Fern jeff[dash]cmk[at]fcse[dot]co[dot]uk for testing and providing feedback to make this addon happen
#
# 2022-04-30: code cleanup/streamlining
#


from typing import List
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    OIDBytes,
    startswith,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringByteTable,
)
from cmk.base.plugins.agent_based.utils.bgp_peer import (
    InvBgpPeer,
    bgp_error_as_string,
    bgp_error_code_as_hex,
    bgp_render_ip_address,
    get_bgp_type,
)
from cmk.base.plugins.agent_based.utils.juniper_bgp_peer import (
    juniper_afi_safi_mapping,
)


def parse_inv_juniper_bgp_peer(string_table: List[StringByteTable]):
    peers, address_families, local_identifier = string_table

    bgp_peers = []

    try:
        local_id = local_identifier[0][0]
    except (ValueError, IndexError):
        local_id = 'N/A'

    peer_families = {}
    for entry in address_families:
        index, afi, safi = entry[0].split('.')
        if peer_families.get(index):
            peer_families[index] = peer_families[index] + [juniper_afi_safi_mapping(afi, safi)]
        else:
            peer_families[index] = [juniper_afi_safi_mapping(afi, safi)]

    for entry in peers:
        oid_end, remote_id, state, negotiated_version, local_addr_type, local_addr, local_as, remote_addr_type, \
        remote_addr, remote_as, peer_index, routing_instance, fsm_established_time, fsm_established_trans, \
        last_error_received = entry

        # assuming IPv4 address format
        remote_id = '.'.join([str(m) for m in remote_id])

        bgp_peer: InvBgpPeer = {
            'remote_addr': bgp_render_ip_address(remote_addr_type, remote_addr),
            'remote_id': remote_id,
            'version': negotiated_version,
            'local_addr': bgp_render_ip_address(local_addr_type, local_addr),
            'remote_as': remote_as,
            'local_as': local_as,
            'local_id': local_id,
            'bgp_type': get_bgp_type(local_as, remote_as),
            'fsm_established_time': int(fsm_established_time),
            'peer_state': 1 if state == '6' else 2,  # adjust to match if_oper_status for inventory painter
            'last_error_code': bgp_error_code_as_hex(last_error_received),
            'last_error': bgp_error_as_string(last_error_received),
            'status_columns': {},
            'address_family': ', '.join(peer_families.get(peer_index, []))
        }

        bgp_peers.append(bgp_peer)

    return bgp_peers


register.snmp_section(
    name='inv_juniper_bgp_peer',
    parse_function=parse_inv_juniper_bgp_peer,
    parsed_section_name='inv_bgp_peer',
    supersedes=['inv_bgp_peer'],
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.2636.5.1.1.2',  # BGP4-V2-MIB-JUNIPER::jnxBgpM2Peer
            oids=[
                OIDEnd(),
                OIDBytes('1.1.1.1'),  # jnxBgpM2PeerIdentifier
                '1.1.1.2',  # jnxBgpM2PeerState
                # '1.1.1.3',  # jnxBgpM2PeerStatus
                '1.1.1.4',  # jnxBgpM2PeerConfiguredVersion
                # '1.1.1.5',  # jnxBgpM2PeerNegotiatedVersion --> is Null '0' :-(
                '1.1.1.6',  # jnxBgpM2PeerLocalAddrType
                OIDBytes('1.1.1.7'),  # jnxBgpM2PeerLocalAddr
                # '1.1.1.8',  # jnxBgpM2PeerLocalPort
                '1.1.1.9',  # jnxBgpM2PeerLocalAs
                '1.1.1.10',  # jnxBgpM2PeerRemoteAddrType
                OIDBytes('1.1.1.11'),  # jnxBgpM2PeerRemoteAddr
                # '1.1.1.12',  # jnxBgpM2PeerRemotePort
                '1.1.1.13',  # jnxBgpM2PeerRemoteAs
                '1.1.1.14',  # jnxBgpM2PeerIndex
                '1.1.1.15',  # jnxBgpM2PeerRoutingInstance
                '4.1.1.1',  # jnxBgpM2PeerFsmEstablishedTime
                '6.1.1.5',  # jnxBgpM2PeerFsmEstablishedTrans
                OIDBytes('2.1.1.1'),  # jnxBgpM2PeerLastErrorReceived
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2636.5.1.1.2.6.2.1',  # BGP4-V2-MIB-JUNIPER::jnxBgpM2PrefixCountersEntry
            oids=[
                OIDEnd(),  # index.afi.safi
                # '1',  # jnxBgpM2PrefixCountersAfi
                # '2',  # jnxBgpM2PrefixCountersSafi
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.2636.5.1.1.1',  # BGP4-V2-MIB-JUNIPER::nxBgpM2BaseScalars
            oids=[
                '6',  # jnxBgpM2LocalIdentifier
            ]
        )
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2636.1.1.1'),
),
